/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formcycle.common.protobuf;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Type;

import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpOutputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.AbstractGenericHttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.http.converter.HttpMessageNotWritableException;
import org.springframework.stereotype.Component;

import com.google.protobuf.CodedOutputStream;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import lombok.extern.log4j.Log4j2;

@Component
@Log4j2
public class ProtobufMessageConverter extends AbstractGenericHttpMessageConverter<Object> {

	public ProtobufMessageConverter() {
		super(new MediaType(com.google.common.net.MediaType.PROTOBUF.type(), com.google.common.net.MediaType.PROTOBUF.subtype()));
	}

	@Override
	protected void writeInternal(Object t, Type type, HttpOutputMessage outputMessage) throws IOException, HttpMessageNotWritableException {
		try {
			var writeMethod = Class.forName(type.getTypeName()).getMethod("writeTo", CodedOutputStream.class);
			CodedOutputStream out = CodedOutputStream.newInstance(outputMessage.getBody());
			writeMethod.invoke(t, out);
			out.flush();
		} catch (NoSuchMethodException | SecurityException | ClassNotFoundException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			LOG.error("Error writing response", e);
			throw new TechnicalException("Error writing response.", e);
		}

	}

	@Override
	public Object read(Type type, Class<?> contextClass, HttpInputMessage inputMessage) throws IOException, HttpMessageNotReadableException {
		try {
			return readInternal(Class.forName(type.getTypeName()), inputMessage);
		} catch (HttpMessageNotReadableException | ClassNotFoundException | IOException e) {
			LOG.error("Error getting target class", e);
			throw new TechnicalException("Error getting target class", e);
		}
	}

	@Override
	protected Object readInternal(Class<? extends Object> clazz, HttpInputMessage inputMessage) throws IOException, HttpMessageNotReadableException {
		try {
			var parserMethod = clazz.getMethod("parseFrom", InputStream.class);
			return parserMethod.invoke(null, inputMessage.getBody());
		} catch (NoSuchMethodException | SecurityException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			LOG.error("Error on reading protobuf data.", e);
			throw new TechnicalException("Error on reading protobuf data.", e);
		}
	}
}
