/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import static de.ozgcloud.eingang.formsolutions.FormSolutionsAttachmentsMapper.*;
import static de.ozgcloud.eingang.formsolutions.FormSolutionsFilesTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.itvsh.kop.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import lombok.SneakyThrows;

class FormSolutionsAttachmentsMapperTest {
	private FormSolutionsAttachmentsMapper mapper = new FormSolutionsAttachmentsMapper();

	@Nested
	class TestAttachmentsMapping {
		@Test
		@SneakyThrows
		void shouldParseZip() {
			var map = mapper.mapAttachments(FormSolutionsFilesTestFactory.create().getFormData());

			assertThat(TestUtils.contentStreamToByteArray(getAttachment(map).getContentStream())).isEqualTo(ZIP_DECODED);
		}

		@Test
		void shouldSetContentType() {
			var map = mapper.mapAttachments(FormSolutionsFilesTestFactory.create().getFormData());

			assertThat(getAttachment(map).getContentType()).isEqualTo(ZIP_CONTENT_TYPE);
		}

		@Test
		void shouldSetFileName() {
			var map = mapper.mapAttachments(FormSolutionsFilesTestFactory.create().getFormData());

			assertThat(getAttachment(map).getName()).isEqualTo(FILE_NAME_ZIP_ATTACHMENT);
		}

		@Test
		void shouldSetGroupName() {
			var map = mapper.mapAttachments(FormSolutionsFilesTestFactory.create().getFormData());

			assertThat(map.get(0).getName()).isEqualTo(FILE_GROUP_ZIP_NAME);
		}
	}

	private IncomingFile getAttachment(List<IncomingFileGroup> attachments) {
		return attachments.get(0).getFiles().get(0);
	}
}
