/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.forwarder;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.kop.eingangsadapter.forwarding.GrpcRouteForwardingResponse;
import de.ozgcloud.eingang.router.GrpcEingangMapper;
import io.grpc.stub.StreamObserver;

class RouteForwardingGrpcServiceTest {

	@InjectMocks
	private RouteForwardingGrpcService service;

	@Mock
	private StreamObserver<GrpcRouteForwardingResponse> responseObserver;

	@Mock
	private RouteForwardingService routeService;
	@Spy
	private RouteCriteriaMapper criteriaMapper = Mappers.getMapper(RouteCriteriaMapper.class);
	@Spy
	private GrpcEingangMapper eingangMapper = Mappers.getMapper(GrpcEingangMapper.class);

	@Nested
	class TestRouteForwarding {

		@Captor
		private ArgumentCaptor<RouteCriteria> criteriaCaptor;

		@Test
		void shouldCallService() {
			service.routeForwarding(GrpcRouteForwardingRequestTestFactory.create(), responseObserver);

			verify(routeService).route(any(), any());
		}

		@Test
		void shouldHaveRouteCriteria() {
			service.routeForwarding(GrpcRouteForwardingRequestTestFactory.create(), responseObserver);

			verify(routeService).route(criteriaCaptor.capture(), any());

			assertThat(criteriaCaptor.getValue()).usingRecursiveComparison().isEqualTo(RouteCriteriaTestFactory.create());
		}

		@Test
		void shouldHaveFormData() {
			service.routeForwarding(GrpcRouteForwardingRequestTestFactory.create(), responseObserver);

			verify(routeService).route(any(), notNull());
			verify(eingangMapper).toFormData(any());
		}
	}

}
