package de.ozgcloud.eingang.xta;

import java.util.Spliterators;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.eingang.common.formdata.FormData;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class XtaService {

	static final String DFOERDERMITTELANTRAG_MESSAGE_TYPE = "Geschaeftsgang.Geschaeftsgang.0201";

	@Autowired
	private XtaRemoteService remoteService;
	@Autowired
	private XtaMessageMapper mapper;

	public Stream<FormData> getMessages() {
		return createXtaMessageStream().filter(this::filterByMessageType).map(this::getFormData);
	}

	Stream<XtaMessageMetaData> createXtaMessageStream() {
		var iterator = new XtaMessageMetadataRemoteIterator(remoteService);
		return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 0), false);
	}

	// filter criteria for dFoerdermittelantrag
	// https://jira.mgm-tp.com/jira/browse/OZG-3659
	boolean filterByMessageType(XtaMessageMetaData metaData) {
		if (StringUtils.equals(metaData.getMessageType(), DFOERDERMITTELANTRAG_MESSAGE_TYPE)) {
			return true;
		}

		LOG.warn("Ignoring XTA-Message of type '{}'.", metaData.getMessageType());
		return false;
	}

	public FormData getFormData(@NonNull XtaMessageMetaData metaData) {
		var msg = remoteService.getMessage(metaData.getMessageId());
		return mapper.toFormData(msg.toBuilder().metaData(metaData).build());
	}

	public void acknowledgeReceive(@NonNull XtaMessageId messageId) {
		remoteService.close(messageId);
	}
}
